; Main Loop
; Copyright 2002 by Thorsten Klose
; Commercial use without my permission is strictly forbidden!

MainLoop
	;; feed the watchdog
	;************************clrwdt
	clrf	MIDI_EXPBYTES			; clear number of expected bytes register

Buffer_LoopReset

	clrf	MIDI_TIMEOUT_LOW		; load timeout counter
	clrf	MIDI_TIMEOUT_HIGH
	movlw	0xe0
	movwf	MIDI_TIMEOUT_HHIGH

Buffer_Loop
	;***********************clrwdt		; feed the watchdog

	;; check for timeout
	incf	MIDI_TIMEOUT_LOW, F 
	skpnz
	incf	MIDI_TIMEOUT_HIGH, F
	skpnz
	incf	MIDI_TIMEOUT_HHIGH, F
	
	movf	MIDI_TIMEOUT_LOW, W
	bnz	Buffer_Loop_NoTimeOut
	movf	MIDI_TIMEOUT_HIGH, W
	bnz	Buffer_Loop_NoTimeOut
	movf	MIDI_TIMEOUT_HHIGH, W
	bz	Buffer_Loop_TimeOut	;; Time Out !!!

Buffer_Loop_NoTimeOut
	;; check for new/next byte in Ringbuffer
	movf	RINGBUFFER0_A, W
	xorwf	RINGBUFFER0_B, W
	bnz	Buffer_ReceiveHandler

	;; if no new byte: loop if waiting for complete MIDI event
	movf	MIDI_EXPBYTES, W
	bnz	Buffer_Loop

	goto	Buffer_Loop_End	; no new value and not waiting for complete event

Buffer_ReceiveHandler	
	;; get byte of SUART Ringbuffer
	incf	RINGBUFFER0_B, F
	movf	RINGBUFFER0_B, W
	xorlw	RINGBUFFER0_END+1
	bnz	Buffer_NoReset
	movlw	RINGBUFFER0
	movwf	RINGBUFFER0_B	
Buffer_NoReset
	movf	RINGBUFFER0_B, W
	movwf	FSR
	movf	INDF, W
	movwf	MIDI_IN

	;; analyse incoming byte
	IFSET	MIDI_IN, 7, goto Buffer_GotCommand

	movf	MIDI0_RUNSTATUS, W	; on sysex: don't take care for the expected bytes
	xorlw	0xf0
	bz	Buffer_ForwardSysEx

	movf	MIDI_EXPBYTES, W	; send byte of running status if necessary
	bnz	Buffer_RunningData

	movf	MIDI0_RUNSTATUS, W
	call	MIDI_GetCMax
	movwf	MIDI_EXPBYTES

	movf	MIDI0_RUNSTATUS, W	; new running status - store it
	call	MIDI_LearnEvent0

Buffer_RunningData
	decf	MIDI_EXPBYTES, F
Buffer_ForwardSysEx
	decf	MIDI_EXPBYTES, W	; MIDI_EXPBYTES == 1?
	bnz	Buffer_NotData1
	movf	MIDI_IN, W
	call	MIDI_LearnEvent1	; learn event 1
Buffer_NotData1
	movf	MIDI_EXPBYTES, W	; MIDI_EXPBYTES == 0?
	bnz	Buffer_NotData2
	movf	MIDI_IN, W
	call	MIDI_LearnEvent2	; learn event 2
Buffer_NotData2
	goto	Buffer_SendEnd

Buffer_GotCommand
	;; Command >= 0xf8?
	movf	INDF, W
	andlw	0xf8
	xorlw	0xf8
	bnz	Buffer_GotCommandLTF8

Buffer_GotSystemCommand
	;; System >= F8: don't change Running Status and number of expected bytes!
	movf	INDF, W			; get received byte
	call	MIDI_LearnEvent0	; 
	goto	Buffer_SendEnd

Buffer_GotCommandLTF8
	;; command less than F8: store running status, send byte, calc number of expected bytes
	movf	INDF, W			; get received byte
	movwf	MIDI0_RUNSTATUS
	call	MIDI_LearnEvent0

	movf	INDF, W			; get number of expected bytes
	call	MIDI_GetCMax
	movwf	MIDI_EXPBYTES		; store it in MIDI_EXPBYTES
	
Buffer_SendEnd

	;; reset Time Out counter and loop again
	goto	Buffer_LoopReset
	;; we got a Time Out?
Buffer_Loop_TimeOut
#if 1
	movlw	0xff
	call	MIDI_SendByte
#endif
	
Buffer_Loop_End

	goto	MainLoop 
