;
; MIDI Processor
;
;  Filter all MIDI events which are sent on Channel 15
;
; Copyright 2001 by Thorsten Klose
; Commercial use without my permission is strictly forbidden!
;

;; --------------------------------------------------------------------------
;;  PROC Received 8n: called on a Note Off Event	OFF
;;  In:	MIDI_LASTEVENT0: 8n, n = midi channel
;;      MIDI_LASTEVENT1: note number
;;      MIDI_LASTEVENT2: velocity
;; --------------------------------------------------------------------------
PROC_Received8n
	;; check MIDI Channel, leave routine if channel != 15
	movf	MIDI_LASTEVENT0, W	; get event byte #0
	andlw	0x0f			; extract channel
	xorlw	0x0f			; compare with 15
	skpz				; skip next command if equal
	return				; leave routine
					; else (if compare was true) continue here
	movf MIDI_LASTEVENT1, w		; get note #
	iorlw	0x80			; set bit 7
	movwf	PORTB			; send it out PORTB
	return

;; --------------------------------------------------------------------------
;;  PROC Received 9n: called on a Note On Event		ON
;;  In:	MIDI_LASTEVENT0: 9n, n = midi channel
;;      MIDI_LASTEVENT1: note number
;;      MIDI_LASTEVENT2: velocity
;; --------------------------------------------------------------------------
PROC_Received9n
	;; check MIDI Channel, leave routine if channel != 15
	movf	MIDI_LASTEVENT0, W	; get event byte #0
	andlw	0x0f			; extract channel
	xorlw	0x0f			; compare with 15
	skpz				; skip next command if equal to 0
	return				; leave routine
					; else (if compare was true) continue here
	movf	MIDI_LASTEVENT2, W	; get velocity
	xorlw 0x00			; compare with 0
	bnz Not_Zero			; velocity != 0, is a note on

	movf	MIDI_LASTEVENT1, W	; get note #, this is a note off since vel = 0
	iorlw 0x80			; set bit 7
	movwf	PORTB			; send it out PORTB
	return

Not_Zero
	movf	MIDI_LASTEVENT1, W	; get note #
	movwf	PORTB			; send it out PORTB
	return



;; --------------------------------------------------------------------------
;;  PROC Received An: called on an Aftertouch Event
;;  In:	MIDI_LASTEVENT0: An, n = midi channel
;;      MIDI_LASTEVENT1: note number
;;      MIDI_LASTEVENT2: preasure
;; --------------------------------------------------------------------------
PROC_ReceivedAn
	return

;; --------------------------------------------------------------------------
;;  PROC Received Bn: called on a Controller Event
;;  In:	MIDI_LASTEVENT0: Bn, n = midi channel
;;      MIDI_LASTEVENT1: CC number
;;      MIDI_LASTEVENT2: CC value
;; --------------------------------------------------------------------------
PROC_ReceivedBn
	return

;; --------------------------------------------------------------------------
;;  PROC Received Cn: called on a Controller Event
;;  In:	MIDI_LASTEVENT0: Cn, n = midi channel
;;      MIDI_LASTEVENT2: program number (AND NOT MIDI_LASTEVENT1!)
;; --------------------------------------------------------------------------
PROC_ReceivedCn
	return

;; --------------------------------------------------------------------------
;;  PROC Received Dn: called on a Poly Aftertouch Event
;;  In:	MIDI_LASTEVENT0: Dn, n = midi channel
;;      MIDI_LASTEVENT2: note number (AND NOT MIDI_LASTEVENT1!)
;; --------------------------------------------------------------------------
PROC_ReceivedDn
	return

;; --------------------------------------------------------------------------
;;  PROC Received En: called on an Pitch Bender Event
;;  In:	MIDI_LASTEVENT0: En, n = midi channel
;;      MIDI_LASTEVENT1: Pitch Bender High Byte
;;      MIDI_LASTEVENT2: Pitch Bender Low Byte
;; --------------------------------------------------------------------------
PROC_ReceivedEn
	return

;; --------------------------------------------------------------------------
;;  PROC Received Fx: called on an System Event
;;  In:	MIDI_LASTEVENT0: Fx, x = System Service
;;      MIDI_LASTEVENT1: contains the byte of a sysex or MTC stream when
;;                       MIDI_FxSTATUS != 0
;;      MIDI_FxSTATUS:   if 0, send MIDI_LASTEVENT0, else MIDI_LASTEVENT1
;; --------------------------------------------------------------------------
PROC_ReceivedFx
	return
